# This is an automatically generated file.
# DO NOT EDIT or your changes may be overwritten
from __future__ import annotations

import base64
from typing import List

from xdrlib3 import Packer, Unpacker

from .dependent_tx_cluster import DependentTxCluster

__all__ = ["ParallelTxExecutionStage"]


class ParallelTxExecutionStage:
    """
    XDR Source Code::

        typedef DependentTxCluster ParallelTxExecutionStage<>;
    """

    def __init__(self, parallel_tx_execution_stage: List[DependentTxCluster]) -> None:
        _expect_max_length = 4294967295
        if (
            parallel_tx_execution_stage
            and len(parallel_tx_execution_stage) > _expect_max_length
        ):
            raise ValueError(
                f"The maximum length of `parallel_tx_execution_stage` should be {_expect_max_length}, but got {len(parallel_tx_execution_stage)}."
            )
        self.parallel_tx_execution_stage = parallel_tx_execution_stage

    def pack(self, packer: Packer) -> None:
        packer.pack_uint(len(self.parallel_tx_execution_stage))
        for parallel_tx_execution_stage_item in self.parallel_tx_execution_stage:
            parallel_tx_execution_stage_item.pack(packer)

    @classmethod
    def unpack(cls, unpacker: Unpacker) -> ParallelTxExecutionStage:
        length = unpacker.unpack_uint()
        parallel_tx_execution_stage = []
        for _ in range(length):
            parallel_tx_execution_stage.append(DependentTxCluster.unpack(unpacker))
        return cls(parallel_tx_execution_stage)

    def to_xdr_bytes(self) -> bytes:
        packer = Packer()
        self.pack(packer)
        return packer.get_buffer()

    @classmethod
    def from_xdr_bytes(cls, xdr: bytes) -> ParallelTxExecutionStage:
        unpacker = Unpacker(xdr)
        return cls.unpack(unpacker)

    def to_xdr(self) -> str:
        xdr_bytes = self.to_xdr_bytes()
        return base64.b64encode(xdr_bytes).decode()

    @classmethod
    def from_xdr(cls, xdr: str) -> ParallelTxExecutionStage:
        xdr_bytes = base64.b64decode(xdr.encode())
        return cls.from_xdr_bytes(xdr_bytes)

    def __hash__(self):
        return hash(self.parallel_tx_execution_stage)

    def __eq__(self, other: object):
        if not isinstance(other, self.__class__):
            return NotImplemented
        return self.parallel_tx_execution_stage == other.parallel_tx_execution_stage

    def __repr__(self):
        return f"<ParallelTxExecutionStage [parallel_tx_execution_stage={self.parallel_tx_execution_stage}]>"



# This is an automatically generated file.
# DO NOT EDIT or your changes may be overwritten
from __future__ import annotations

import base64
from typing import Optional

from xdrlib3 import Packer, Unpacker

from .base import Integer
from .ledger_close_meta_v0 import LedgerCloseMetaV0
from .ledger_close_meta_v1 import LedgerCloseMetaV1
from .ledger_close_meta_v2 import LedgerCloseMetaV2

__all__ = ["LedgerCloseMeta"]


class LedgerCloseMeta:
    """
    XDR Source Code::

        union LedgerCloseMeta switch (int v)
        {
        case 0:
            LedgerCloseMetaV0 v0;
        case 1:
            LedgerCloseMetaV1 v1;
        case 2:
            LedgerCloseMetaV2 v2;
        };
    """

    def __init__(
        self,
        v: int,
        v0: Optional[LedgerCloseMetaV0] = None,
        v1: Optional[LedgerCloseMetaV1] = None,
        v2: Optional[LedgerCloseMetaV2] = None,
    ) -> None:
        self.v = v
        self.v0 = v0
        self.v1 = v1
        self.v2 = v2

    def pack(self, packer: Packer) -> None:
        Integer(self.v).pack(packer)
        if self.v == 0:
            if self.v0 is None:
                raise ValueError("v0 should not be None.")
            self.v0.pack(packer)
            return
        if self.v == 1:
            if self.v1 is None:
                raise ValueError("v1 should not be None.")
            self.v1.pack(packer)
            return
        if self.v == 2:
            if self.v2 is None:
                raise ValueError("v2 should not be None.")
            self.v2.pack(packer)
            return

    @classmethod
    def unpack(cls, unpacker: Unpacker) -> LedgerCloseMeta:
        v = Integer.unpack(unpacker)
        if v == 0:
            v0 = LedgerCloseMetaV0.unpack(unpacker)
            return cls(v=v, v0=v0)
        if v == 1:
            v1 = LedgerCloseMetaV1.unpack(unpacker)
            return cls(v=v, v1=v1)
        if v == 2:
            v2 = LedgerCloseMetaV2.unpack(unpacker)
            return cls(v=v, v2=v2)
        return cls(v=v)

    def to_xdr_bytes(self) -> bytes:
        packer = Packer()
        self.pack(packer)
        return packer.get_buffer()

    @classmethod
    def from_xdr_bytes(cls, xdr: bytes) -> LedgerCloseMeta:
        unpacker = Unpacker(xdr)
        return cls.unpack(unpacker)

    def to_xdr(self) -> str:
        xdr_bytes = self.to_xdr_bytes()
        return base64.b64encode(xdr_bytes).decode()

    @classmethod
    def from_xdr(cls, xdr: str) -> LedgerCloseMeta:
        xdr_bytes = base64.b64decode(xdr.encode())
        return cls.from_xdr_bytes(xdr_bytes)

    def __hash__(self):
        return hash(
            (
                self.v,
                self.v0,
                self.v1,
                self.v2,
            )
        )

    def __eq__(self, other: object):
        if not isinstance(other, self.__class__):
            return NotImplemented
        return (
            self.v == other.v
            and self.v0 == other.v0
            and self.v1 == other.v1
            and self.v2 == other.v2
        )

    def __repr__(self):
        out = []
        out.append(f"v={self.v}")
        out.append(f"v0={self.v0}") if self.v0 is not None else None
        out.append(f"v1={self.v1}") if self.v1 is not None else None
        out.append(f"v2={self.v2}") if self.v2 is not None else None
        return f"<LedgerCloseMeta [{', '.join(out)}]>"



# This is an automatically generated file.
# DO NOT EDIT or your changes may be overwritten
from __future__ import annotations

import base64
from typing import Optional

from xdrlib3 import Packer, Unpacker

from .sc_val import SCVal
from .soroban_transaction_meta_ext import SorobanTransactionMetaExt

__all__ = ["SorobanTransactionMetaV2"]


class SorobanTransactionMetaV2:
    """
    XDR Source Code::

        struct SorobanTransactionMetaV2
        {
            SorobanTransactionMetaExt ext;

            SCVal* returnValue;
        };
    """

    def __init__(
        self,
        ext: SorobanTransactionMetaExt,
        return_value: Optional[SCVal],
    ) -> None:
        self.ext = ext
        self.return_value = return_value

    def pack(self, packer: Packer) -> None:
        self.ext.pack(packer)
        if self.return_value is None:
            packer.pack_uint(0)
        else:
            packer.pack_uint(1)
            self.return_value.pack(packer)

    @classmethod
    def unpack(cls, unpacker: Unpacker) -> SorobanTransactionMetaV2:
        ext = SorobanTransactionMetaExt.unpack(unpacker)
        return_value = SCVal.unpack(unpacker) if unpacker.unpack_uint() else None
        return cls(
            ext=ext,
            return_value=return_value,
        )

    def to_xdr_bytes(self) -> bytes:
        packer = Packer()
        self.pack(packer)
        return packer.get_buffer()

    @classmethod
    def from_xdr_bytes(cls, xdr: bytes) -> SorobanTransactionMetaV2:
        unpacker = Unpacker(xdr)
        return cls.unpack(unpacker)

    def to_xdr(self) -> str:
        xdr_bytes = self.to_xdr_bytes()
        return base64.b64encode(xdr_bytes).decode()

    @classmethod
    def from_xdr(cls, xdr: str) -> SorobanTransactionMetaV2:
        xdr_bytes = base64.b64decode(xdr.encode())
        return cls.from_xdr_bytes(xdr_bytes)

    def __hash__(self):
        return hash(
            (
                self.ext,
                self.return_value,
            )
        )

    def __eq__(self, other: object):
        if not isinstance(other, self.__class__):
            return NotImplemented
        return self.ext == other.ext and self.return_value == other.return_value

    def __repr__(self):
        out = [
            f"ext={self.ext}",
            f"return_value={self.return_value}",
        ]
        return f"<SorobanTransactionMetaV2 [{', '.join(out)}]>"



# This is an automatically generated file.
# DO NOT EDIT or your changes may be overwritten
from __future__ import annotations

import base64
from typing import List

from xdrlib3 import Packer, Unpacker

from .ledger_close_meta import LedgerCloseMeta
from .uint32 import Uint32

__all__ = ["LedgerCloseMetaBatch"]


class LedgerCloseMetaBatch:
    """
    XDR Source Code::

        struct LedgerCloseMetaBatch
        {
            // starting ledger sequence number in the batch
            uint32 startSequence;

            // ending ledger sequence number in the batch
            uint32 endSequence;

            // Ledger close meta for each ledger within the batch
            LedgerCloseMeta ledgerCloseMetas<>;
        };
    """

    def __init__(
        self,
        start_sequence: Uint32,
        end_sequence: Uint32,
        ledger_close_metas: List[LedgerCloseMeta],
    ) -> None:
        _expect_max_length = 4294967295
        if ledger_close_metas and len(ledger_close_metas) > _expect_max_length:
            raise ValueError(
                f"The maximum length of `ledger_close_metas` should be {_expect_max_length}, but got {len(ledger_close_metas)}."
            )
        self.start_sequence = start_sequence
        self.end_sequence = end_sequence
        self.ledger_close_metas = ledger_close_metas

    def pack(self, packer: Packer) -> None:
        self.start_sequence.pack(packer)
        self.end_sequence.pack(packer)
        packer.pack_uint(len(self.ledger_close_metas))
        for ledger_close_metas_item in self.ledger_close_metas:
            ledger_close_metas_item.pack(packer)

    @classmethod
    def unpack(cls, unpacker: Unpacker) -> LedgerCloseMetaBatch:
        start_sequence = Uint32.unpack(unpacker)
        end_sequence = Uint32.unpack(unpacker)
        length = unpacker.unpack_uint()
        ledger_close_metas = []
        for _ in range(length):
            ledger_close_metas.append(LedgerCloseMeta.unpack(unpacker))
        return cls(
            start_sequence=start_sequence,
            end_sequence=end_sequence,
            ledger_close_metas=ledger_close_metas,
        )

    def to_xdr_bytes(self) -> bytes:
        packer = Packer()
        self.pack(packer)
        return packer.get_buffer()

    @classmethod
    def from_xdr_bytes(cls, xdr: bytes) -> LedgerCloseMetaBatch:
        unpacker = Unpacker(xdr)
        return cls.unpack(unpacker)

    def to_xdr(self) -> str:
        xdr_bytes = self.to_xdr_bytes()
        return base64.b64encode(xdr_bytes).decode()

    @classmethod
    def from_xdr(cls, xdr: str) -> LedgerCloseMetaBatch:
        xdr_bytes = base64.b64decode(xdr.encode())
        return cls.from_xdr_bytes(xdr_bytes)

    def __hash__(self):
        return hash(
            (
                self.start_sequence,
                self.end_sequence,
                self.ledger_close_metas,
            )
        )

    def __eq__(self, other: object):
        if not isinstance(other, self.__class__):
            return NotImplemented
        return (
            self.start_sequence == other.start_sequence
            and self.end_sequence == other.end_sequence
            and self.ledger_close_metas == other.ledger_close_metas
        )

    def __repr__(self):
        out = [
            f"start_sequence={self.start_sequence}",
            f"end_sequence={self.end_sequence}",
            f"ledger_close_metas={self.ledger_close_metas}",
        ]
        return f"<LedgerCloseMetaBatch [{', '.join(out)}]>"



# This is an automatically generated file.
# DO NOT EDIT or your changes may be overwritten
from __future__ import annotations

import base64
from typing import List, Optional

from xdrlib3 import Packer, Unpacker

from .int64 import Int64
from .parallel_tx_execution_stage import ParallelTxExecutionStage

__all__ = ["ParallelTxsComponent"]


class ParallelTxsComponent:
    """
    XDR Source Code::

        struct ParallelTxsComponent
        {
          int64* baseFee;
          // A sequence of stages that *may* have arbitrary data dependencies between
          // each other, i.e. in a general case the stage execution order may not be
          // arbitrarily shuffled without affecting the end result.
          ParallelTxExecutionStage executionStages<>;
        };
    """

    def __init__(
        self,
        base_fee: Optional[Int64],
        execution_stages: List[ParallelTxExecutionStage],
    ) -> None:
        _expect_max_length = 4294967295
        if execution_stages and len(execution_stages) > _expect_max_length:
            raise ValueError(
                f"The maximum length of `execution_stages` should be {_expect_max_length}, but got {len(execution_stages)}."
            )
        self.base_fee = base_fee
        self.execution_stages = execution_stages

    def pack(self, packer: Packer) -> None:
        if self.base_fee is None:
            packer.pack_uint(0)
        else:
            packer.pack_uint(1)
            self.base_fee.pack(packer)
        packer.pack_uint(len(self.execution_stages))
        for execution_stages_item in self.execution_stages:
            execution_stages_item.pack(packer)

    @classmethod
    def unpack(cls, unpacker: Unpacker) -> ParallelTxsComponent:
        base_fee = Int64.unpack(unpacker) if unpacker.unpack_uint() else None
        length = unpacker.unpack_uint()
        execution_stages = []
        for _ in range(length):
            execution_stages.append(ParallelTxExecutionStage.unpack(unpacker))
        return cls(
            base_fee=base_fee,
            execution_stages=execution_stages,
        )

    def to_xdr_bytes(self) -> bytes:
        packer = Packer()
        self.pack(packer)
        return packer.get_buffer()

    @classmethod
    def from_xdr_bytes(cls, xdr: bytes) -> ParallelTxsComponent:
        unpacker = Unpacker(xdr)
        return cls.unpack(unpacker)

    def to_xdr(self) -> str:
        xdr_bytes = self.to_xdr_bytes()
        return base64.b64encode(xdr_bytes).decode()

    @classmethod
    def from_xdr(cls, xdr: str) -> ParallelTxsComponent:
        xdr_bytes = base64.b64decode(xdr.encode())
        return cls.from_xdr_bytes(xdr_bytes)

    def __hash__(self):
        return hash(
            (
                self.base_fee,
                self.execution_stages,
            )
        )

    def __eq__(self, other: object):
        if not isinstance(other, self.__class__):
            return NotImplemented
        return (
            self.base_fee == other.base_fee
            and self.execution_stages == other.execution_stages
        )

    def __repr__(self):
        out = [
            f"base_fee={self.base_fee}",
            f"execution_stages={self.execution_stages}",
        ]
        return f"<ParallelTxsComponent [{', '.join(out)}]>"



# This is an automatically generated file.
# DO NOT EDIT or your changes may be overwritten
from __future__ import annotations

import base64

from xdrlib3 import Packer, Unpacker

from .int64 import Int64
from .uint32 import Uint32

__all__ = ["ConfigSettingContractLedgerCostExtV0"]


class ConfigSettingContractLedgerCostExtV0:
    """
    XDR Source Code::

        struct ConfigSettingContractLedgerCostExtV0
        {
            // Maximum number of RO+RW entries in the transaction footprint.
            uint32 txMaxFootprintEntries;
            // Fee per 1 KB of data written to the ledger.
            // Unlike the rent fee, this is a flat fee that is charged for any ledger
            // write, independent of the type of the entry being written.
            int64 feeWrite1KB;
        };
    """

    def __init__(
        self,
        tx_max_footprint_entries: Uint32,
        fee_write1_kb: Int64,
    ) -> None:
        self.tx_max_footprint_entries = tx_max_footprint_entries
        self.fee_write1_kb = fee_write1_kb

    def pack(self, packer: Packer) -> None:
        self.tx_max_footprint_entries.pack(packer)
        self.fee_write1_kb.pack(packer)

    @classmethod
    def unpack(cls, unpacker: Unpacker) -> ConfigSettingContractLedgerCostExtV0:
        tx_max_footprint_entries = Uint32.unpack(unpacker)
        fee_write1_kb = Int64.unpack(unpacker)
        return cls(
            tx_max_footprint_entries=tx_max_footprint_entries,
            fee_write1_kb=fee_write1_kb,
        )

    def to_xdr_bytes(self) -> bytes:
        packer = Packer()
        self.pack(packer)
        return packer.get_buffer()

    @classmethod
    def from_xdr_bytes(cls, xdr: bytes) -> ConfigSettingContractLedgerCostExtV0:
        unpacker = Unpacker(xdr)
        return cls.unpack(unpacker)

    def to_xdr(self) -> str:
        xdr_bytes = self.to_xdr_bytes()
        return base64.b64encode(xdr_bytes).decode()

    @classmethod
    def from_xdr(cls, xdr: str) -> ConfigSettingContractLedgerCostExtV0:
        xdr_bytes = base64.b64decode(xdr.encode())
        return cls.from_xdr_bytes(xdr_bytes)

    def __hash__(self):
        return hash(
            (
                self.tx_max_footprint_entries,
                self.fee_write1_kb,
            )
        )

    def __eq__(self, other: object):
        if not isinstance(other, self.__class__):
            return NotImplemented
        return (
            self.tx_max_footprint_entries == other.tx_max_footprint_entries
            and self.fee_write1_kb == other.fee_write1_kb
        )

    def __repr__(self):
        out = [
            f"tx_max_footprint_entries={self.tx_max_footprint_entries}",
            f"fee_write1_kb={self.fee_write1_kb}",
        ]
        return f"<ConfigSettingContractLedgerCostExtV0 [{', '.join(out)}]>"



# This is an automatically generated file.
# DO NOT EDIT or your changes may be overwritten
from __future__ import annotations

import base64

from xdrlib3 import Packer, Unpacker

from .uint32 import Uint32

__all__ = ["ConfigSettingSCPTiming"]


class ConfigSettingSCPTiming:
    """
    XDR Source Code::

        struct ConfigSettingSCPTiming {
            uint32 ledgerTargetCloseTimeMilliseconds;
            uint32 nominationTimeoutInitialMilliseconds;
            uint32 nominationTimeoutIncrementMilliseconds;
            uint32 ballotTimeoutInitialMilliseconds;
            uint32 ballotTimeoutIncrementMilliseconds;
        };
    """

    def __init__(
        self,
        ledger_target_close_time_milliseconds: Uint32,
        nomination_timeout_initial_milliseconds: Uint32,
        nomination_timeout_increment_milliseconds: Uint32,
        ballot_timeout_initial_milliseconds: Uint32,
        ballot_timeout_increment_milliseconds: Uint32,
    ) -> None:
        self.ledger_target_close_time_milliseconds = (
            ledger_target_close_time_milliseconds
        )
        self.nomination_timeout_initial_milliseconds = (
            nomination_timeout_initial_milliseconds
        )
        self.nomination_timeout_increment_milliseconds = (
            nomination_timeout_increment_milliseconds
        )
        self.ballot_timeout_initial_milliseconds = ballot_timeout_initial_milliseconds
        self.ballot_timeout_increment_milliseconds = (
            ballot_timeout_increment_milliseconds
        )

    def pack(self, packer: Packer) -> None:
        self.ledger_target_close_time_milliseconds.pack(packer)
        self.nomination_timeout_initial_milliseconds.pack(packer)
        self.nomination_timeout_increment_milliseconds.pack(packer)
        self.ballot_timeout_initial_milliseconds.pack(packer)
        self.ballot_timeout_increment_milliseconds.pack(packer)

    @classmethod
    def unpack(cls, unpacker: Unpacker) -> ConfigSettingSCPTiming:
        ledger_target_close_time_milliseconds = Uint32.unpack(unpacker)
        nomination_timeout_initial_milliseconds = Uint32.unpack(unpacker)
        nomination_timeout_increment_milliseconds = Uint32.unpack(unpacker)
        ballot_timeout_initial_milliseconds = Uint32.unpack(unpacker)
        ballot_timeout_increment_milliseconds = Uint32.unpack(unpacker)
        return cls(
            ledger_target_close_time_milliseconds=ledger_target_close_time_milliseconds,
            nomination_timeout_initial_milliseconds=nomination_timeout_initial_milliseconds,
            nomination_timeout_increment_milliseconds=nomination_timeout_increment_milliseconds,
            ballot_timeout_initial_milliseconds=ballot_timeout_initial_milliseconds,
            ballot_timeout_increment_milliseconds=ballot_timeout_increment_milliseconds,
        )

    def to_xdr_bytes(self) -> bytes:
        packer = Packer()
        self.pack(packer)
        return packer.get_buffer()

    @classmethod
    def from_xdr_bytes(cls, xdr: bytes) -> ConfigSettingSCPTiming:
        unpacker = Unpacker(xdr)
        return cls.unpack(unpacker)

    def to_xdr(self) -> str:
        xdr_bytes = self.to_xdr_bytes()
        return base64.b64encode(xdr_bytes).decode()

    @classmethod
    def from_xdr(cls, xdr: str) -> ConfigSettingSCPTiming:
        xdr_bytes = base64.b64decode(xdr.encode())
        return cls.from_xdr_bytes(xdr_bytes)

    def __hash__(self):
        return hash(
            (
                self.ledger_target_close_time_milliseconds,
                self.nomination_timeout_initial_milliseconds,
                self.nomination_timeout_increment_milliseconds,
                self.ballot_timeout_initial_milliseconds,
                self.ballot_timeout_increment_milliseconds,
            )
        )

    def __eq__(self, other: object):
        if not isinstance(other, self.__class__):
            return NotImplemented
        return (
            self.ledger_target_close_time_milliseconds
            == other.ledger_target_close_time_milliseconds
            and self.nomination_timeout_initial_milliseconds
            == other.nomination_timeout_initial_milliseconds
            and self.nomination_timeout_increment_milliseconds
            == other.nomination_timeout_increment_milliseconds
            and self.ballot_timeout_initial_milliseconds
            == other.ballot_timeout_initial_milliseconds
            and self.ballot_timeout_increment_milliseconds
            == other.ballot_timeout_increment_milliseconds
        )

    def __repr__(self):
        out = [
            f"ledger_target_close_time_milliseconds={self.ledger_target_close_time_milliseconds}",
            f"nomination_timeout_initial_milliseconds={self.nomination_timeout_initial_milliseconds}",
            f"nomination_timeout_increment_milliseconds={self.nomination_timeout_increment_milliseconds}",
            f"ballot_timeout_initial_milliseconds={self.ballot_timeout_initial_milliseconds}",
            f"ballot_timeout_increment_milliseconds={self.ballot_timeout_increment_milliseconds}",
        ]
        return f"<ConfigSettingSCPTiming [{', '.join(out)}]>"



# This is an automatically generated file.
# DO NOT EDIT or your changes may be overwritten
from __future__ import annotations

import base64
from typing import List

from xdrlib3 import Packer, Unpacker

from .soroban_authorization_entry import SorobanAuthorizationEntry

__all__ = ["SorobanAuthorizationEntries"]


class SorobanAuthorizationEntries:
    """
    XDR Source Code::

        typedef SorobanAuthorizationEntry SorobanAuthorizationEntries<>;
    """

    def __init__(
        self, soroban_authorization_entries: List[SorobanAuthorizationEntry]
    ) -> None:
        _expect_max_length = 4294967295
        if (
            soroban_authorization_entries
            and len(soroban_authorization_entries) > _expect_max_length
        ):
            raise ValueError(
                f"The maximum length of `soroban_authorization_entries` should be {_expect_max_length}, but got {len(soroban_authorization_entries)}."
            )
        self.soroban_authorization_entries = soroban_authorization_entries

    def pack(self, packer: Packer) -> None:
        packer.pack_uint(len(self.soroban_authorization_entries))
        for soroban_authorization_entries_item in self.soroban_authorization_entries:
            soroban_authorization_entries_item.pack(packer)

    @classmethod
    def unpack(cls, unpacker: Unpacker) -> SorobanAuthorizationEntries:
        length = unpacker.unpack_uint()
        soroban_authorization_entries = []
        for _ in range(length):
            soroban_authorization_entries.append(
                SorobanAuthorizationEntry.unpack(unpacker)
            )
        return cls(soroban_authorization_entries)

    def to_xdr_bytes(self) -> bytes:
        packer = Packer()
        self.pack(packer)
        return packer.get_buffer()

    @classmethod
    def from_xdr_bytes(cls, xdr: bytes) -> SorobanAuthorizationEntries:
        unpacker = Unpacker(xdr)
        return cls.unpack(unpacker)

    def to_xdr(self) -> str:
        xdr_bytes = self.to_xdr_bytes()
        return base64.b64encode(xdr_bytes).decode()

    @classmethod
    def from_xdr(cls, xdr: str) -> SorobanAuthorizationEntries:
        xdr_bytes = base64.b64decode(xdr.encode())
        return cls.from_xdr_bytes(xdr_bytes)

    def __hash__(self):
        return hash(self.soroban_authorization_entries)

    def __eq__(self, other: object):
        if not isinstance(other, self.__class__):
            return NotImplemented
        return self.soroban_authorization_entries == other.soroban_authorization_entries

    def __repr__(self):
        return f"<SorobanAuthorizationEntries [soroban_authorization_entries={self.soroban_authorization_entries}]>"



# This is an automatically generated file.
# DO NOT EDIT or your changes may be overwritten
from __future__ import annotations

import base64

from xdrlib3 import Packer, Unpacker

from .uint32 import Uint32

__all__ = ["ConfigSettingContractParallelComputeV0"]


class ConfigSettingContractParallelComputeV0:
    """
    XDR Source Code::

        struct ConfigSettingContractParallelComputeV0
        {
            // Maximum number of clusters with dependent transactions allowed in a
            // stage of parallel tx set component.
            // This effectively sets the lower bound on the number of physical threads
            // necessary to effectively apply transaction sets in parallel.
            uint32 ledgerMaxDependentTxClusters;
        };
    """

    def __init__(
        self,
        ledger_max_dependent_tx_clusters: Uint32,
    ) -> None:
        self.ledger_max_dependent_tx_clusters = ledger_max_dependent_tx_clusters

    def pack(self, packer: Packer) -> None:
        self.ledger_max_dependent_tx_clusters.pack(packer)

    @classmethod
    def unpack(cls, unpacker: Unpacker) -> ConfigSettingContractParallelComputeV0:
        ledger_max_dependent_tx_clusters = Uint32.unpack(unpacker)
        return cls(
            ledger_max_dependent_tx_clusters=ledger_max_dependent_tx_clusters,
        )

    def to_xdr_bytes(self) -> bytes:
        packer = Packer()
        self.pack(packer)
        return packer.get_buffer()

    @classmethod
    def from_xdr_bytes(cls, xdr: bytes) -> ConfigSettingContractParallelComputeV0:
        unpacker = Unpacker(xdr)
        return cls.unpack(unpacker)

    def to_xdr(self) -> str:
        xdr_bytes = self.to_xdr_bytes()
        return base64.b64encode(xdr_bytes).decode()

    @classmethod
    def from_xdr(cls, xdr: str) -> ConfigSettingContractParallelComputeV0:
        xdr_bytes = base64.b64decode(xdr.encode())
        return cls.from_xdr_bytes(xdr_bytes)

    def __hash__(self):
        return hash((self.ledger_max_dependent_tx_clusters,))

    def __eq__(self, other: object):
        if not isinstance(other, self.__class__):
            return NotImplemented
        return (
            self.ledger_max_dependent_tx_clusters
            == other.ledger_max_dependent_tx_clusters
        )

    def __repr__(self):
        out = [
            f"ledger_max_dependent_tx_clusters={self.ledger_max_dependent_tx_clusters}",
        ]
        return f"<ConfigSettingContractParallelComputeV0 [{', '.join(out)}]>"



# This is an automatically generated file.
# DO NOT EDIT or your changes may be overwritten
from __future__ import annotations

import base64
from typing import List

from xdrlib3 import Packer, Unpacker

from .base import String
from .constants import *
from .sc_spec_event_data_format import SCSpecEventDataFormat
from .sc_spec_event_param_v0 import SCSpecEventParamV0
from .sc_symbol import SCSymbol

__all__ = ["SCSpecEventV0"]


class SCSpecEventV0:
    """
    XDR Source Code::

        struct SCSpecEventV0
        {
            string doc<SC_SPEC_DOC_LIMIT>;
            string lib<80>;
            SCSymbol name;
            SCSymbol prefixTopics<2>;
            SCSpecEventParamV0 params<50>;
            SCSpecEventDataFormat dataFormat;
        };
    """

    def __init__(
        self,
        doc: bytes,
        lib: bytes,
        name: SCSymbol,
        prefix_topics: List[SCSymbol],
        params: List[SCSpecEventParamV0],
        data_format: SCSpecEventDataFormat,
    ) -> None:
        _expect_max_length = 2
        if prefix_topics and len(prefix_topics) > _expect_max_length:
            raise ValueError(
                f"The maximum length of `prefix_topics` should be {_expect_max_length}, but got {len(prefix_topics)}."
            )
        _expect_max_length = 50
        if params and len(params) > _expect_max_length:
            raise ValueError(
                f"The maximum length of `params` should be {_expect_max_length}, but got {len(params)}."
            )
        self.doc = doc
        self.lib = lib
        self.name = name
        self.prefix_topics = prefix_topics
        self.params = params
        self.data_format = data_format

    def pack(self, packer: Packer) -> None:
        String(self.doc, SC_SPEC_DOC_LIMIT).pack(packer)
        String(self.lib, 80).pack(packer)
        self.name.pack(packer)
        packer.pack_uint(len(self.prefix_topics))
        for prefix_topics_item in self.prefix_topics:
            prefix_topics_item.pack(packer)
        packer.pack_uint(len(self.params))
        for params_item in self.params:
            params_item.pack(packer)
        self.data_format.pack(packer)

    @classmethod
    def unpack(cls, unpacker: Unpacker) -> SCSpecEventV0:
        doc = String.unpack(unpacker)
        lib = String.unpack(unpacker)
        name = SCSymbol.unpack(unpacker)
        length = unpacker.unpack_uint()
        prefix_topics = []
        for _ in range(length):
            prefix_topics.append(SCSymbol.unpack(unpacker))
        length = unpacker.unpack_uint()
        params = []
        for _ in range(length):
            params.append(SCSpecEventParamV0.unpack(unpacker))
        data_format = SCSpecEventDataFormat.unpack(unpacker)
        return cls(
            doc=doc,
            lib=lib,
            name=name,
            prefix_topics=prefix_topics,
            params=params,
            data_format=data_format,
        )

    def to_xdr_bytes(self) -> bytes:
        packer = Packer()
        self.pack(packer)
        return packer.get_buffer()

    @classmethod
    def from_xdr_bytes(cls, xdr: bytes) -> SCSpecEventV0:
        unpacker = Unpacker(xdr)
        return cls.unpack(unpacker)

    def to_xdr(self) -> str:
        xdr_bytes = self.to_xdr_bytes()
        return base64.b64encode(xdr_bytes).decode()

    @classmethod
    def from_xdr(cls, xdr: str) -> SCSpecEventV0:
        xdr_bytes = base64.b64decode(xdr.encode())
        return cls.from_xdr_bytes(xdr_bytes)

    def __hash__(self):
        return hash(
            (
                self.doc,
                self.lib,
                self.name,
                self.prefix_topics,
                self.params,
                self.data_format,
            )
        )

    def __eq__(self, other: object):
        if not isinstance(other, self.__class__):
            return NotImplemented
        return (
            self.doc == other.doc
            and self.lib == other.lib
            and self.name == other.name
            and self.prefix_topics == other.prefix_topics
            and self.params == other.params
            and self.data_format == other.data_format
        )

    def __repr__(self):
        out = [
            f"doc={self.doc}",
            f"lib={self.lib}",
            f"name={self.name}",
            f"prefix_topics={self.prefix_topics}",
            f"params={self.params}",
            f"data_format={self.data_format}",
        ]
        return f"<SCSpecEventV0 [{', '.join(out)}]>"



# This is an automatically generated file.
# DO NOT EDIT or your changes may be overwritten
from __future__ import annotations

import base64

from xdrlib3 import Packer, Unpacker

from .contract_event import ContractEvent
from .transaction_event_stage import TransactionEventStage

__all__ = ["TransactionEvent"]


class TransactionEvent:
    """
    XDR Source Code::

        struct TransactionEvent {
            TransactionEventStage stage;  // Stage at which an event has occurred.
            ContractEvent event;  // The contract event that has occurred.
        };
    """

    def __init__(
        self,
        stage: TransactionEventStage,
        event: ContractEvent,
    ) -> None:
        self.stage = stage
        self.event = event

    def pack(self, packer: Packer) -> None:
        self.stage.pack(packer)
        self.event.pack(packer)

    @classmethod
    def unpack(cls, unpacker: Unpacker) -> TransactionEvent:
        stage = TransactionEventStage.unpack(unpacker)
        event = ContractEvent.unpack(unpacker)
        return cls(
            stage=stage,
            event=event,
        )

    def to_xdr_bytes(self) -> bytes:
        packer = Packer()
        self.pack(packer)
        return packer.get_buffer()

    @classmethod
    def from_xdr_bytes(cls, xdr: bytes) -> TransactionEvent:
        unpacker = Unpacker(xdr)
        return cls.unpack(unpacker)

    def to_xdr(self) -> str:
        xdr_bytes = self.to_xdr_bytes()
        return base64.b64encode(xdr_bytes).decode()

    @classmethod
    def from_xdr(cls, xdr: str) -> TransactionEvent:
        xdr_bytes = base64.b64decode(xdr.encode())
        return cls.from_xdr_bytes(xdr_bytes)

    def __hash__(self):
        return hash(
            (
                self.stage,
                self.event,
            )
        )

    def __eq__(self, other: object):
        if not isinstance(other, self.__class__):
            return NotImplemented
        return self.stage == other.stage and self.event == other.event

    def __repr__(self):
        out = [
            f"stage={self.stage}",
            f"event={self.event}",
        ]
        return f"<TransactionEvent [{', '.join(out)}]>"



# This is an automatically generated file.
# DO NOT EDIT or your changes may be overwritten
from __future__ import annotations

import base64
from typing import List

from xdrlib3 import Packer, Unpacker

from .uint32 import Uint32

__all__ = ["SorobanResourcesExtV0"]


class SorobanResourcesExtV0:
    """
    XDR Source Code::

        struct SorobanResourcesExtV0
        {
            // Vector of indices representing what Soroban
            // entries in the footprint are archived, based on the
            // order of keys provided in the readWrite footprint.
            uint32 archivedSorobanEntries<>;
        };
    """

    def __init__(
        self,
        archived_soroban_entries: List[Uint32],
    ) -> None:
        _expect_max_length = 4294967295
        if (
            archived_soroban_entries
            and len(archived_soroban_entries) > _expect_max_length
        ):
            raise ValueError(
                f"The maximum length of `archived_soroban_entries` should be {_expect_max_length}, but got {len(archived_soroban_entries)}."
            )
        self.archived_soroban_entries = archived_soroban_entries

    def pack(self, packer: Packer) -> None:
        packer.pack_uint(len(self.archived_soroban_entries))
        for archived_soroban_entries_item in self.archived_soroban_entries:
            archived_soroban_entries_item.pack(packer)

    @classmethod
    def unpack(cls, unpacker: Unpacker) -> SorobanResourcesExtV0:
        length = unpacker.unpack_uint()
        archived_soroban_entries = []
        for _ in range(length):
            archived_soroban_entries.append(Uint32.unpack(unpacker))
        return cls(
            archived_soroban_entries=archived_soroban_entries,
        )

    def to_xdr_bytes(self) -> bytes:
        packer = Packer()
        self.pack(packer)
        return packer.get_buffer()

    @classmethod
    def from_xdr_bytes(cls, xdr: bytes) -> SorobanResourcesExtV0:
        unpacker = Unpacker(xdr)
        return cls.unpack(unpacker)

    def to_xdr(self) -> str:
        xdr_bytes = self.to_xdr_bytes()
        return base64.b64encode(xdr_bytes).decode()

    @classmethod
    def from_xdr(cls, xdr: str) -> SorobanResourcesExtV0:
        xdr_bytes = base64.b64decode(xdr.encode())
        return cls.from_xdr_bytes(xdr_bytes)

    def __hash__(self):
        return hash((self.archived_soroban_entries,))

    def __eq__(self, other: object):
        if not isinstance(other, self.__class__):
            return NotImplemented
        return self.archived_soroban_entries == other.archived_soroban_entries

    def __repr__(self):
        out = [
            f"archived_soroban_entries={self.archived_soroban_entries}",
        ]
        return f"<SorobanResourcesExtV0 [{', '.join(out)}]>"



import binascii
from enum import IntEnum
from typing import Union

from . import xdr as stellar_xdr
from .strkey import StrKey

__all__ = ["Address", "AddressType"]


class AddressType(IntEnum):
    """Represents an Address type."""

    ACCOUNT = 0
    """An account address, address looks like ``GBJCHUKZMTFSLOMNC7P4TS4VJJBTCYL3XKSOLXAUJSD56C4LHND5T...``."""

    CONTRACT = 1
    """An contract address, address looks like ``CCJZ5DGASBWQXR5MPFCJXMBI333XE5U3FSJTNQU7RIKE3P5GN2K2W...``."""

    MUXED_ACCOUNT = 2
    """A muxed account address, address looks like ``MAQAA5L65LSYH7CQ3VTJ7F3HHLGCL3DSLAR2Y47263D56MNNGHSQSAAAAAAAAAAE2L...``."""

    CLAIMABLE_BALANCE = 3
    """A claimable balance address, address looks like ``BAAD6DBUX6J22DMZOHIEZTEQ64CVCHEDRKWZONFEUL5Q26QD7R76RGR...``."""

    LIQUIDITY_POOL = 4
    """A liquidity pool address, address looks like ``LA7QYNF7SOWQ3GLR2BGMZEHXAVIRZA4KVWLTJJFC7MGXUA74P7UJU...``."""


class Address:
    """Represents a single address in the Stellar network.
    An address can represent an account or a contract.

    :param address: ID of the account, contract, muxed account, claimable balance, or liquidity pool.
    """

    def __init__(self, address: str):
        if StrKey.is_valid_ed25519_public_key(address):
            self.type = AddressType.ACCOUNT
            self.key = StrKey.decode_ed25519_public_key(address)
        elif StrKey.is_valid_contract(address):
            self.type = AddressType.CONTRACT
            self.key = StrKey.decode_contract(address)
        elif StrKey.is_valid_med25519_public_key(address):
            self.type = AddressType.MUXED_ACCOUNT
            self.key = StrKey.decode_med25519_public_key(address)
        elif StrKey.is_valid_claimable_balance(address):
            self.type = AddressType.CLAIMABLE_BALANCE
            self.key = StrKey.decode_claimable_balance(address)
        elif StrKey.is_valid_liquidity_pool(address):
            self.type = AddressType.LIQUIDITY_POOL
            self.key = StrKey.decode_liquidity_pool(address)
        else:
            raise ValueError("Unsupported address type.")

    @property
    def address(self) -> str:
        """Returns the encoded address.

        :return: The encoded address.
        """
        if self.type == AddressType.ACCOUNT:
            return StrKey.encode_ed25519_public_key(self.key)
        elif self.type == AddressType.CONTRACT:
            return StrKey.encode_contract(self.key)
        elif self.type == AddressType.MUXED_ACCOUNT:
            return StrKey.encode_med25519_public_key(self.key)
        elif self.type == AddressType.CLAIMABLE_BALANCE:
            return StrKey.encode_claimable_balance(self.key)
        elif self.type == AddressType.LIQUIDITY_POOL:
            return StrKey.encode_liquidity_pool(self.key)
        else:
            raise ValueError("Unsupported address type.")

    @staticmethod
    def from_raw_account(account: Union[bytes, str]) -> "Address":
        """Creates a new account Address object from raw bytes.

        :param account: The raw bytes of the account, it can be a byte array or a hex encoded string.
        :return: A new Address object.
        """
        if isinstance(account, str):
            account = binascii.unhexlify(account)
        return Address(StrKey.encode_ed25519_public_key(account))

    @staticmethod
    def from_raw_contract(contract: Union[bytes, str]) -> "Address":
        """Creates a new contract Address object from a buffer of raw bytes.

        :param contract: The raw bytes of the contract.
        :return: A new Address object.
        """
        if isinstance(contract, str):
            contract = binascii.unhexlify(contract)
        return Address(StrKey.encode_contract(contract))

    @staticmethod
    def from_raw_muxed_account(muxed_account: Union[bytes, str]) -> "Address":
        """Creates a new muxed account Address object from raw bytes.

        :param muxed_account: The raw bytes of the muxed account.
        :return: A new Address object.
        """
        if isinstance(muxed_account, str):
            muxed_account = binascii.unhexlify(muxed_account)
        return Address(StrKey.encode_med25519_public_key(muxed_account))

    @staticmethod
    def from_raw_claimable_balance(claimable_balance: Union[bytes, str]) -> "Address":
        """Creates a new claimable balance Address object from raw bytes.

        :param claimable_balance: The raw bytes of the claimable balance.
        :return: A new Address object.
        """
        if isinstance(claimable_balance, str):
            claimable_balance = binascii.unhexlify(claimable_balance)
        return Address(StrKey.encode_claimable_balance(claimable_balance))

    @staticmethod
    def from_raw_liquidity_pool(liquidity_pool: Union[bytes, str]) -> "Address":
        """Creates a new liquidity pool Address object from raw bytes.

        :param liquidity_pool: The raw bytes of the liquidity pool.
        :return: A new Address object.
        """
        if isinstance(liquidity_pool, str):
            liquidity_pool = binascii.unhexlify(liquidity_pool)
        return Address(StrKey.encode_liquidity_pool(liquidity_pool))

    def to_xdr_sc_address(self) -> stellar_xdr.SCAddress:
        """Converts the Address object to a :class:`stellar_sdk.xdr.SCAddress` XDR object.

        :return: A :class:`stellar_sdk.xdr.SCAddress` XDR object.
        """
        if self.type == AddressType.ACCOUNT:
            account = stellar_xdr.AccountID(
                stellar_xdr.PublicKey(
                    stellar_xdr.PublicKeyType.PUBLIC_KEY_TYPE_ED25519,
                    stellar_xdr.Uint256(self.key),
                )
            )
            return stellar_xdr.SCAddress(
                stellar_xdr.SCAddressType.SC_ADDRESS_TYPE_ACCOUNT, account_id=account
            )
        elif self.type == AddressType.CONTRACT:
            contract = stellar_xdr.ContractID.from_xdr_bytes(self.key)
            return stellar_xdr.SCAddress(
                stellar_xdr.SCAddressType.SC_ADDRESS_TYPE_CONTRACT, contract_id=contract
            )
        elif self.type == AddressType.MUXED_ACCOUNT:
            muxed_account = stellar_xdr.MuxedEd25519Account.from_xdr_bytes(self.key)
            return stellar_xdr.SCAddress(
                stellar_xdr.SCAddressType.SC_ADDRESS_TYPE_MUXED_ACCOUNT,
                muxed_account=muxed_account,
            )
        elif self.type == AddressType.CLAIMABLE_BALANCE:
            # See https://github.com/stellar/stellar-protocol/pull/1646/files#r1974431825
            if self.key[:1] != b"\x00":
                raise ValueError(
                    "The claimable balance ID type is not supported, it must be `CLAIMABLE_BALANCE_ID_TYPE_V0`."
                )
            claimable_balance_id = stellar_xdr.ClaimableBalanceID(
                stellar_xdr.ClaimableBalanceIDType.CLAIMABLE_BALANCE_ID_TYPE_V0,
                v0=stellar_xdr.Hash.from_xdr_bytes(self.key[1:]),
            )
            return stellar_xdr.SCAddress(
                stellar_xdr.SCAddressType.SC_ADDRESS_TYPE_CLAIMABLE_BALANCE,
                claimable_balance_id=claimable_balance_id,
            )
        elif self.type == AddressType.LIQUIDITY_POOL:
            liquidity_pool_id = stellar_xdr.PoolID.from_xdr_bytes(self.key)
            return stellar_xdr.SCAddress(
                stellar_xdr.SCAddressType.SC_ADDRESS_TYPE_LIQUIDITY_POOL,
                liquidity_pool_id=liquidity_pool_id,
            )
        else:
            raise ValueError("Unsupported address type.")

    @classmethod
    def from_xdr_sc_address(cls, sc_address: stellar_xdr.SCAddress) -> "Address":
        """Creates a new Address object from a :class:`stellar_sdk.xdr.SCAddress` XDR object.

        :param sc_address: The :class:`stellar_sdk.xdr.SCAddress` XDR object.
        :return: A new Address object.
        """
        if sc_address.type == stellar_xdr.SCAddressType.SC_ADDRESS_TYPE_ACCOUNT:
            assert sc_address.account_id is not None
            assert sc_address.account_id.account_id.ed25519 is not None
            return cls.from_raw_account(
                sc_address.account_id.account_id.ed25519.uint256
            )
        elif sc_address.type == stellar_xdr.SCAddressType.SC_ADDRESS_TYPE_CONTRACT:
            assert sc_address.contract_id is not None
            return cls.from_raw_contract(sc_address.contract_id.to_xdr_bytes())
        elif sc_address.type == stellar_xdr.SCAddressType.SC_ADDRESS_TYPE_MUXED_ACCOUNT:
            assert sc_address.muxed_account is not None
            return cls.from_raw_muxed_account(sc_address.muxed_account.to_xdr_bytes())
        elif (
            sc_address.type
            == stellar_xdr.SCAddressType.SC_ADDRESS_TYPE_CLAIMABLE_BALANCE
        ):
            assert sc_address.claimable_balance_id is not None
            # See https://github.com/stellar/stellar-protocol/pull/1646/files#r1974431825
            if (
                sc_address.claimable_balance_id.type
                != stellar_xdr.ClaimableBalanceIDType.CLAIMABLE_BALANCE_ID_TYPE_V0
            ):
                raise ValueError(
                    f"The claimable balance ID type is not supported: {sc_address.claimable_balance_id.type.name}"
                )  # This is a safeguard.
            return cls.from_raw_claimable_balance(
                sc_address.claimable_balance_id.to_xdr_bytes()[3:]
            )
        elif (
            sc_address.type == stellar_xdr.SCAddressType.SC_ADDRESS_TYPE_LIQUIDITY_POOL
        ):
            assert sc_address.liquidity_pool_id is not None
            return cls.from_raw_liquidity_pool(
                sc_address.liquidity_pool_id.to_xdr_bytes()
            )
        else:
            raise ValueError("Unsupported address type.")

    def to_xdr_sc_val(self) -> stellar_xdr.SCVal:
        return stellar_xdr.SCVal(
            stellar_xdr.SCValType.SCV_ADDRESS, address=self.to_xdr_sc_address()
        )

    @classmethod
    def from_xdr_sc_val(cls, sc_val: stellar_xdr.SCVal) -> "Address":
        if sc_val.type != stellar_xdr.SCValType.SCV_ADDRESS:
            raise ValueError("Unsupported SCVal type.")
        assert sc_val.address is not None
        return cls.from_xdr_sc_address(sc_val.address)

    def __hash__(self):
        return hash((self.key, self.type))

    def __eq__(self, other: object) -> bool:
        if not isinstance(other, self.__class__):
            return NotImplemented
        return self.key == other.key and self.type == other.type

    def __repr__(self):
        return f"<Address [type={self.type.name}, address={self.address}]>"



# This is an automatically generated file.
# DO NOT EDIT or your changes may be overwritten
from __future__ import annotations

import base64

from xdrlib3 import Packer, Unpacker

from .time_point import TimePoint

__all__ = ["TimeBounds"]


class TimeBounds:
    """
    XDR Source Code::

        struct TimeBounds
        {
            TimePoint minTime;
            TimePoint maxTime; // 0 here means no maxTime
        };
    """

    def __init__(
        self,
        min_time: TimePoint,
        max_time: TimePoint,
    ) -> None:
        self.min_time = min_time
        self.max_time = max_time

    def pack(self, packer: Packer) -> None:
        self.min_time.pack(packer)
        self.max_time.pack(packer)

    @classmethod
    def unpack(cls, unpacker: Unpacker) -> TimeBounds:
        min_time = TimePoint.unpack(unpacker)
        max_time = TimePoint.unpack(unpacker)
        return cls(
            min_time=min_time,
            max_time=max_time,
        )

    def to_xdr_bytes(self) -> bytes:
        packer = Packer()
        self.pack(packer)
        return packer.get_buffer()

    @classmethod
    def from_xdr_bytes(cls, xdr: bytes) -> TimeBounds:
        unpacker = Unpacker(xdr)
        return cls.unpack(unpacker)

    def to_xdr(self) -> str:
        xdr_bytes = self.to_xdr_bytes()
        return base64.b64encode(xdr_bytes).decode()

    @classmethod
    def from_xdr(cls, xdr: str) -> TimeBounds:
        xdr_bytes = base64.b64decode(xdr.encode())
        return cls.from_xdr_bytes(xdr_bytes)

    def __hash__(self):
        return hash(
            (
                self.min_time,
                self.max_time,
            )
        )

    def __eq__(self, other: object):
        if not isinstance(other, self.__class__):
            return NotImplemented
        return self.min_time == other.min_time and self.max_time == other.max_time

    def __repr__(self):
        out = [
            f"min_time={self.min_time}",
            f"max_time={self.max_time}",
        ]
        return f"<TimeBounds [{', '.join(out)}]>"
